<?php

/**
 * @file
 * Admin settings callbacks for no referrer module.
 */

/**
 * Admin settings form.
 */
function noreferrer_admin() {
  $form['noreferrer_link'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Add <code>rel="noreferrer"</code> to themed links'),
    '#default_value' => variable_get('noreferrer_link', TRUE),
    '#description'   => t('If checked, the <code>rel="noreferrer"</code> attribute will be added to non-whitelisted links generated by code (e.g. menu module). Enabling this setting may have a negative impact on site performance because each link will be processed by the theme layer.'),
  );
  $form['noreferrer_whitelisted_domains'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Whitelisted domains'),
    '#default_value' => variable_get('noreferrer_whitelisted_domains', ''),
    '#description'   => t('Enter a space-separated list of domains to which referrer URLs will be sent (e.g. <em>example.com example.org</em>). Links to all other domains will have a <code>rel="noreferrer"</code> attribute added.'),
    '#maxlength'     => NULL,
  );
  $form['noreferrer_publish'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Publish list of whitelisted domains'),
    '#default_value' => variable_get('noreferrer_publish', FALSE),
    '#description'   => t('If checked, the list of whitelisted domains will be published at %url when saving this form.', array(
      '%url' => file_create_url(noreferrer_publish_uri()),
    )),
  );
  $form['noreferrer_subscribe_url'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Subscribe to external list of whitelisted domains'),
    '#default_value' => variable_get('noreferrer_subscribe_url', ''),
    '#description'   => t('If configured, the list of whitelisted domains will be retrieved from the given URL during each cron run.'),
  );
  $form = system_settings_form($form);
  $form['#submit'][] = 'noreferrer_admin_submit';
  return $form;
}

/**
 * Admin settings validate handler.
 */
function noreferrer_admin_validate($form, &$form_state) {
  if (strlen($form_state['values']['noreferrer_subscribe_url']) && !valid_url($form_state['values']['noreferrer_subscribe_url'], TRUE)) {
    form_set_error('noreferrer_subscribe_url', t('Please provide a valid URL.'));
  }
}

/**
 * Admin settings submit handler.
 */
function noreferrer_admin_submit($form, &$form_state) {
  drupal_register_shutdown_function('drupal_flush_all_caches');
  if (variable_get('noreferrer_publish', FALSE)) {
    noreferrer_publish();
  }
  if ($url = variable_get('noreferrer_subscribe_url', '')) {
    noreferrer_subscribe($url);
  }
}

/**
 * Publishes domain whitelist.
 */
function noreferrer_publish() {
  if ($whitelist = variable_get('noreferrer_whitelisted_domains', '')) {
    $whitelist = drupal_json_encode(explode(' ', $whitelist));
    file_unmanaged_save_data($whitelist, noreferrer_publish_uri(), FILE_EXISTS_REPLACE);
  }
}

/**
 * Returns domain whitelist URI.
 */
function noreferrer_publish_uri() {
  // For security through obscurity purposes, the whitelist URL is secret.
  return 'public://noreferrer-whitelist-' . drupal_hmac_base64('noreferrer-whitelist', drupal_get_private_key()) . '.json';
}

/**
 * Retrieves whitelist from external URL.
 */
function noreferrer_subscribe($url) {
  $response = drupal_http_request($url);
  if (!empty($response->error)) {
    watchdog('noreferrer', 'Error %code received at %url while retrieving domain whitelist: %error.', array(
      '%code' => $response->code,
      '%url' => $url,
      '%error' => $response->error,
    ), WATCHDOG_ERROR, l(t('Configure No Referrer settings.'), 'admin/config/search/noreferrer'));
  }
  elseif ($response->code == 200 && ($whitelist = drupal_json_decode($response->data)) && is_array($whitelist)) {
    variable_set('noreferrer_whitelisted_domains', implode(' ', $whitelist));
  }
  else {
    watchdog('noreferrer', 'Unable to extract valid data from %url while retrieving domain whitelist.', array(
      '%url' => $url,
    ), WATCHDOG_ERROR, l(t('Configure No Referrer settings.'), 'admin/config/search/noreferrer'));
  }
}
